import arcpy
import numpy as np
import os
import sys
import time
import math

def progressBar(value, endvalue, bar_length=20):
        percent = float(value) / endvalue
        arrow = '-' * int(round(percent * bar_length)-1) + '>'
        spaces = ' ' * (bar_length - len(arrow))

        sys.stdout.write("\rWorking: [{0}] {1}%".format(arrow + spaces, int(round(percent * 100))))
        sys.stdout.flush()
		
def normalize(v, tolerance=0.00001):
    mag2 = sum(n * n for n in v)
    if abs(mag2 - 1.0) > tolerance:
        mag = sqrt(mag2)
        v = tuple(n / mag for n in v)
    return v
	
def q_mult(q1, q2):
    w1, x1, y1, z1 = q1
    w2, x2, y2, z2 = q2
    w = w1 * w2 - x1 * x2 - y1 * y2 - z1 * z2
    x = w1 * x2 + x1 * w2 + y1 * z2 - z1 * y2
    y = w1 * y2 + y1 * w2 + z1 * x2 - x1 * z2
    z = w1 * z2 + z1 * w2 + x1 * y2 - y1 * x2
    return w, x, y, z

def q_conjugate(q):
    w, x, y, z = q
    return (w, -x, -y, -z)

def qv_mult(q1, v1):
    q2 = (0.0,) + v1
    return q_mult(q_mult(q1, q2), q_conjugate(q1))[1:]

def axisangle_to_q(v, theta):
    v = normalize(v)
    x, y, z = v
    theta /= 2
    w = math.cos(theta)
    x = x * math.sin(theta)
    y = y * math.sin(theta)
    z = z * math.sin(theta)
    return w, x, y, z
	
def q_to_axisangle(q):
    w, v = q[0], q[1:]
    theta = acos(w) * 2.0
    return normalize(v), theta

def rotation_matrix(axis, angle):
    s = math.sin(angle)
    c = math.cos(angle)
	
    ux = axis[0]
    uy = axis[1]
    uz = axis[2]
	
    R = np.matrix([
    [c+((ux**2)*(1-c)),         ((ux*uy)*(1-c))-(uz*s),    ((ux*uz)*(1-c))+(uy*s)],
    [((uy*ux)*(1-c))+(uz*s),    c+((uy**2)*(1-c)),         ((uy*uz)*(1-c))-(ux*s)],
    [((uz*ux)*(1-c))-(uy*s),    ((uz*uy)*(1-c))+(ux*s),    c+((uz**2)*(1-c))]])
	
    return R
	


A = np.load("gudena_np_A.npy")
Z = np.load("gudena_np_Z.npy")
X = np.concatenate((A[:,0:2],Z), axis=1)
num_points = X.shape[0]
center = X.sum(axis=0)/num_points
center = np.expand_dims(center, axis=0)
#np.save("gudena_np_A", A)
#np.save("gudena_np_Z", Z)
#print "Took " + time.clock() - str(start_time), "seconds"
plane_constants = np.dot(np.linalg.inv(np.dot(A.T,A)),np.dot(A.T,Z))

#normal = np.concatenate((plane_constants[0:2,:],np.array([-1])), axis=1)

normal = np.array([plane_constants[0,0],plane_constants[1,0],-1])
horiz_plane_normal = np.array([0,0,1])

print(normal)
print(horiz_plane_normal)

normal_length = np.linalg.norm(normal)
horiz_plane_normal_length = np.linalg.norm(horiz_plane_normal)


cos_theta = np.dot(normal, horiz_plane_normal) / (normal_length*horiz_plane_normal_length)
angle = math.acos(cos_theta)
axis = np.cross(normal, horiz_plane_normal)
axis = axis/np.linalg.norm(axis)
print(angle)
print(axis)

cos_half_theta = math.cos(angle/2)
sin_half_theta = math.sin(angle/2)

#q = np.array([cos_half_theta, sin_half_theta*axis[0], sin_half_theta*axis[1], sin_half_theta*axis[2]])
test_R = rotation_matrix(np.array([1,0,0]), np.pi/2)
print("test_R")
print(test_R)


R = rotation_matrix(axis, angle)

testrot = np.dot(test_R,np.array([0,1,1]))
print(testrot)

print(R)
print(R.shape)
print(X.shape)

XT = X.T
XT_norm = XT - center.T
rotated_X = np.dot(R, XT_norm)
rotated_X = rotated_X + center.T

print(rotated_X.shape)

pt = arcpy.Point()
ptGeoms = []
_prog = -1

print(center)
center_rot = rotated_X.T.sum(axis=0)/num_points
print(center_rot)
for idx in range(0, num_points):
    prog = (idx*100)/num_points
    p = rotated_X[:, idx]

#    r1 = axisangle_to_q((axis[0], axis[1], axis[2]), angle)
#    v = qv_mult(r1, (p[0], p[1], p[2]))

    #print(p)
    #print(p.shape)
    pt.X = p[0,0]
    pt.Y = p[1,0]
    pt.Z = p[2,0]
    ptGeoms.append(arcpy.PointGeometry(pt))
    if prog > _prog:
        progressBar(idx, num_points)
    
#    print(str(idx) + "/" + str(num_points))
    _prog = prog
	
print("done.")
arcpy.CopyFeatures_management(ptGeoms, "gudena_rotated.shp")
#print(".")
		

